@@ -33,8 +33,11 @@ class ContactsParserObject : public QObject
     Q_OBJECT
 
 public:
-    ContactsParserObject(QByteArray data, QObject *parent = 0);
+    ContactsParserObject(QByteArray data,
+                         const QString strAuth,
+                         QObject *parent = 0);
     void setEmitLog (bool enable = true);
+    ~ContactsParserObject();
 
 signals:
     //! Status emitter for status bar
@@ -47,10 +50,50 @@ signals:
 public slots:
     void doWork ();
 
+private slots:
+    void onGotOneContact (const ContactInfo &contactInfo);
+    void onGotOnePhoto (const ContactInfo &contactInfo);
+
+private:
+    QNetworkRequest createRequest(QString strUrl);
+    void decRef (bool rv = true);
+
 private:
     QByteArray  byData;
 
     bool        bEmitLog;
+
+    QNetworkAccessManager *nwMgr;
+    QString               strGoogleAuth;
+
+    QAtomicInt  refCount;
+};
+
+class PhotoReplyTracker : public QObject
+{
+    Q_OBJECT
+
+public:
+    PhotoReplyTracker(const ContactInfo &ci,
+                            QNetworkReply *r,
+                            QObject *parent = NULL);
+
+signals:
+    // Emitted when the photo is retrieved from the contact
+    void gotOneContact (const ContactInfo &contactInfo);
+
+public slots:
+    void onFinished();
+
+private slots:
+    void onResponseTimeout();
+
+private:
+    QNetworkReply  *reply;
+    ContactInfo     contactInfo;
+    QTimer          responseTimeout;
+
+    bool            aborted;
 };
 
 #endif // CONTACTSPARSEROBJECT_H
